@@ -1,4 +1,5 @@
 from hashlib import sha1
+
 from ..util import compat
 from ..util import langhelpers
 
@@ -23,43 +24,47 @@ def function_key_generator(namespace, fn, to_str=compat.string_type):
     """
 
     if namespace is None:
-        namespace = '%s:%s' % (fn.__module__, fn.__name__)
+        namespace = "%s:%s" % (fn.__module__, fn.__name__)
     else:
-        namespace = '%s:%s|%s' % (fn.__module__, fn.__name__, namespace)
+        namespace = "%s:%s|%s" % (fn.__module__, fn.__name__, namespace)
 
     args = compat.inspect_getargspec(fn)
-    has_self = args[0] and args[0][0] in ('self', 'cls')
+    has_self = args[0] and args[0][0] in ("self", "cls")
 
     def generate_key(*args, **kw):
         if kw:
             raise ValueError(
                 "dogpile.cache's default key creation "
-                "function does not accept keyword arguments.")
+                "function does not accept keyword arguments."
+            )
         if has_self:
             args = args[1:]
 
         return namespace + "|" + " ".join(map(to_str, args))
+
     return generate_key
 
 
 def function_multi_key_generator(namespace, fn, to_str=compat.string_type):
 
     if namespace is None:
-        namespace = '%s:%s' % (fn.__module__, fn.__name__)
+        namespace = "%s:%s" % (fn.__module__, fn.__name__)
     else:
-        namespace = '%s:%s|%s' % (fn.__module__, fn.__name__, namespace)
+        namespace = "%s:%s|%s" % (fn.__module__, fn.__name__, namespace)
 
     args = compat.inspect_getargspec(fn)
-    has_self = args[0] and args[0][0] in ('self', 'cls')
+    has_self = args[0] and args[0][0] in ("self", "cls")
 
     def generate_keys(*args, **kw):
         if kw:
             raise ValueError(
                 "dogpile.cache's default key creation "
-                "function does not accept keyword arguments.")
+                "function does not accept keyword arguments."
+            )
         if has_self:
             args = args[1:]
         return [namespace + "|" + key for key in map(to_str, args)]
+
     return generate_keys
 
 
@@ -83,9 +88,9 @@ def kwarg_function_key_generator(namespace, fn, to_str=compat.string_type):
     """
 
     if namespace is None:
-        namespace = '%s:%s' % (fn.__module__, fn.__name__)
+        namespace = "%s:%s" % (fn.__module__, fn.__name__)
     else:
-        namespace = '%s:%s|%s' % (fn.__module__, fn.__name__, namespace)
+        namespace = "%s:%s|%s" % (fn.__module__, fn.__name__, namespace)
 
     argspec = compat.inspect_getargspec(fn)
     default_list = list(argspec.defaults or [])
@@ -94,32 +99,41 @@ def kwarg_function_key_generator(namespace, fn, to_str=compat.string_type):
     # enumerate()
     default_list.reverse()
     # use idx*-1 to create the correct right-lookup index.
-    args_with_defaults = dict((argspec.args[(idx*-1)], default)
-                              for idx, default in enumerate(default_list, 1))
-    if argspec.args and argspec.args[0] in ('self', 'cls'):
+    args_with_defaults = dict(
+        (argspec.args[(idx * -1)], default)
+        for idx, default in enumerate(default_list, 1)
+    )
+    if argspec.args and argspec.args[0] in ("self", "cls"):
         arg_index_start = 1
     else:
         arg_index_start = 0
 
     def generate_key(*args, **kwargs):
         as_kwargs = dict(
-            [(argspec.args[idx], arg)
-             for idx, arg in enumerate(args[arg_index_start:],
-                                       arg_index_start)])
+            [
+                (argspec.args[idx], arg)
+                for idx, arg in enumerate(
+                    args[arg_index_start:], arg_index_start
+                )
+            ]
+        )
         as_kwargs.update(kwargs)
         for arg, val in args_with_defaults.items():
             if arg not in as_kwargs:
                 as_kwargs[arg] = val
 
-        argument_values = [as_kwargs[key]
-                           for key in sorted(as_kwargs.keys())]
-        return namespace + '|' + " ".join(map(to_str, argument_values))
+        argument_values = [as_kwargs[key] for key in sorted(as_kwargs.keys())]
+        return namespace + "|" + " ".join(map(to_str, argument_values))
+
     return generate_key
 
 
 def sha1_mangle_key(key):
     """a SHA1 key mangler."""
 
+    if isinstance(key, compat.text_type):
+        key = key.encode("utf-8")
+
     return sha1(key).hexdigest()
 
 
@@ -128,13 +142,16 @@ def length_conditional_mangler(length, mangler):
     past a certain threshold.
 
     """
+
     def mangle(key):
         if len(key) >= length:
             return mangler(key)
         else:
             return key
+
     return mangle
 
+
 # in the 0.6 release these functions were moved to the dogpile.util namespace.
 # They are linked here to maintain compatibility with older versions.
 
@@ -143,3 +160,30 @@ KeyReentrantMutex = langhelpers.KeyReentrantMutex
 memoized_property = langhelpers.memoized_property
 PluginLoader = langhelpers.PluginLoader
 to_list = langhelpers.to_list
+
+
+class repr_obj(object):
+
+    __slots__ = ("value", "max_chars")
+
+    def __init__(self, value, max_chars=300):
+        self.value = value
+        self.max_chars = max_chars
+
+    def __eq__(self, other):
+        return other.value == self.value
+
+    def __repr__(self):
+        rep = repr(self.value)
+        lenrep = len(rep)
+        if lenrep > self.max_chars:
+            segment_length = self.max_chars // 2
+            rep = (
+                rep[0:segment_length]
+                + (
+                    " ... (%d characters truncated) ... "
+                    % (lenrep - self.max_chars)
+                )
+                + rep[-segment_length:]
+            )
+        return rep
